/*
 * Copyright 2018 Chris Cromer
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * The main Sernatur namespace
 */
namespace Sernatur {
	using Postgres;
	using LibConfig;

	/**
	 * The MainWindow class
	 */
	[GtkTemplate (ui = "/cl/cromer/ubb/sernatur/sernatur.window.ui")]
	public class MainWindow : Gtk.ApplicationWindow {

		/**
		* Initialize the main window class
		* @param application The application used to make the GLib object
		*/
		public MainWindow (Gtk.Application application) {
			GLib.Object (application: application);
			Database conn;

			var file = File.new_for_path ("/etc/sernatur/sernatur.conf");
			if (!file.query_exists ()) {
				stderr.printf (dgettext (null, "The file '%s' doesn't exist.\n"), file.get_path ());
				application.quit ();
				return;
			}

			Config config = Config();
			if (!config.read_file ("/etc/sernatur/sernatur.conf")) {
				stderr.printf ("%s\n", config.error_message ());
				application.quit ();
				return;
			}

			Setting setting = config.lookup("database");
			string* host;
			int port;
			string* options;
			string* gtty;
			string* db_name;
			string* login;
			string* pwd;
			setting.lookup_string("host", out host);
			setting.lookup_int("port", out port);
			setting.lookup_string("options", out options);
			setting.lookup_string("gtty", out gtty);
			setting.lookup_string("db_name", out db_name);
			setting.lookup_string("login", out login);
			setting.lookup_string("pwd", out pwd);

			Settings settings = new Settings ("cl.cromer.ubb.sernatur");
			var host2 = settings.get_string ("host");

			conn = set_db_login (host2, port.to_string (), options, gtty, db_name, login, pwd);
			if (conn.get_status () != ConnectionStatus.OK) {
				stderr.printf ("%s\n", conn.get_error_message ());
				Gtk.MessageDialog msg = new Gtk.MessageDialog (this,
					Gtk.DialogFlags.MODAL,
					Gtk.MessageType.ERROR,
					Gtk.ButtonsType.CLOSE,
					conn.get_error_message ());
				msg.response.connect ((response_id) => {
					switch (response_id) {
						case Gtk.ResponseType.CLOSE:
							application.quit ();
							break;
						case Gtk.ResponseType.DELETE_EVENT:
							application.quit ();
							break;
					}
					msg.destroy();
				});
				msg.set_title (dgettext (null, "Error"));
				msg.show ();
				return;
			}

			/*Result res = conn.exec ("CREATE DATABASE bdd;");
			if (res.get_status () != ExecStatus.COMMAND_OK) {
				stderr.printf ("%s\n", conn.get_error_message ());
				application.quit ();
				return;
			}

			string password = encrypt_password ("bdd", "bdd");

			res = conn.exec ("CREATE USER bdd WITH ENCRYPTED PASSWORD '" + password + "';");
			if (res.get_status () != ExecStatus.COMMAND_OK) {
				stderr.printf ("%s\n", conn.get_error_message ());
				application.quit ();
				return;
			}

			res = conn.exec ("GRANT ALL PRIVILEGES ON DATABASE bdd TO bdd;");
			if (res.get_status () != ExecStatus.COMMAND_OK) {
				stderr.printf ("%s\n", conn.get_error_message ());
				application.quit ();
				return;
			}*/

			GLib.print (dgettext (null, "Server version:") + " %d\n", conn.get_server_version ());
		}
	}
}
